### A Pluto.jl notebook ###
# v0.19.38

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
end

# ╔═╡ b72215bb-8e3a-46e2-8817-271befe1178c
begin
	using PlotlyBase, HypertextLiteral, PlutoUI, PlutoPlotly, LinearAlgebra , NLsolve
	using Dates, PeriodicalDates , DataFrames, CSV, Statistics, PlutoTeachingTools
	import PlotlyJS: savefig
end

# ╔═╡ 3d4a86c6-9638-4e7c-86fb-f2764cd74c9f
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 82090c38-1ecd-43eb-aeca-135d996b8e72
md"""
# Week 1 - Welcome to Pluto Notebooks

## Introduction to Pluto

**Macroeconomics, ISCTE-IUL**
"""

# ╔═╡ 76164b77-2e0b-4deb-8ef8-1b2263260e7a
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas**

**January 2024**
"""

# ╔═╡ 694d78c7-4762-4d40-8f9a-6c3d16916d4e
begin
	function Base.show(io::IO, mimetype::MIME"text/html", plt::PlotlyBase.Plot)
       # Remove responsive flag on the plot as we handle responsibity via ResizeObeserver and leaving it on makes the div flickr during updates
	hasproperty(plt,:config) && plt.config.responsive && (plt.config.responsive = false)   
	show(io,mimetype, @htl("""
			<div>
			<script id=asdf>
			const {plotly} = await import("https://cdn.plot.ly/plotly-2.2.0.min.js")
			const PLOT = this ?? document.createElement("div");
		

		
			Plotly.react(PLOT, $(HypertextLiteral.JavaScript(PlotlyBase.json(plt))));


		
			const pluto_output = currentScript.parentElement.closest('pluto-output')

			const resizeObserver = new ResizeObserver(entries => {
				Plotly.Plots.resize(PLOT)
			})

			resizeObserver.observe(pluto_output)

			invalidation.then(() => {
				resizeObserver.disconnect()
			})
		
			return PLOT
			</script>
			</div>
	"""))
	end
end

# ╔═╡ 554b2cc5-99e1-4eda-b080-67ab2eddac16
md"""
### Packages used in this notebook
"""

# ╔═╡ 31023dd0-f304-471e-a292-2a40915ebcbf
md"""
## 1. What is a Pluto notebook?
"""

# ╔═╡ 7afb249a-ee89-4874-91fa-6da606948b2a
md"""
A notebook is a document with two types of cells (see image below): 
- code cells
- markdown cells 

The first type of cells includes computational code, comments, and Pluto's "begin-end" blocks. Code cells produce an output which usually takes the form of a plot, a table, numbers, or symbols.

The second type of cell (markdown) serves to write standard text, mathematics, illustrate computational code, insert an image or a video, among other functionalities. 

The versatility that arises from the combination of markdown cells with code cells renders notebooks a remarkable tool in computational work in general and teaching in particular.  
"""

# ╔═╡ 072b3431-0b22-4801-a75e-25cd77a7d70d
Resource("https://vivaldomendes.org/images/depot/Pluto_cells.png", :width=>850)

# ╔═╡ 86f1359d-3637-4892-9720-e042fae6543a
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 0fc80d2d-6519-4f17-83c4-25ab9b26d8ce
md"""
## 2. Pluto basic tips
"""

# ╔═╡ 16ecc97b-a401-4172-bcbb-b0baa156b53c
md"""
#### Opening a new cell
"""

# ╔═╡ 564ff460-aa1f-4b9d-abf3-a024a13d96a3
md"""

Put the mouse on top of any opened cell. You will see a $+$ sign on the top left corner, and also on the bottom left corner. Click on the $+$ sign where you want the new cell to be placed.
"""

# ╔═╡ 6307472b-3a3e-4d9a-82fd-25d295abaae1
md"""
#### How to run a cell?

- Click on the small circle-arrow on the bottom right hand-side of each cell (▶)
- Or, just click simultaneosly on `shift` and `enter`
"""

# ╔═╡ 638ac59a-6fb2-447f-8856-0c6f74494e7b
md"""
#### How to save a notebook?

- Click simultaneosly on `Ctrl` and `S`

"""

# ╔═╡ 98f18fc3-0748-4ec7-a6a0-955e2ea22e19
md"""
#### Turn a new cell into markdown

"""

# ╔═╡ e81d6184-2b39-4205-83a8-9e2d7e5d41dd
md"""
Put the cursor inside the cell you want to be in markdown. Press simultaneously the following: `Ctrl` and `M`. That is all. You can start typing text or mathematics as you like.

"""

# ╔═╡ 70af87bd-81b5-4aa0-b1f7-0639d1f09d0a
md"""
!!! tip
	To write text in bold, italics, or mathematical symbols do the following: 
	- type: `**This is bold**` to get: **This is bold**
	- type: `_This is italics_` to get: _This is italics_
	- type: `**_This is bold and italics_**` to get: **_This is bold and italics_**
	- type: `$y=2x^2$` to get a mathematical expression: $y=2x^2$
"""

# ╔═╡ 2da4b605-2d9e-494a-810f-5d153dee7096
md"""
#### Bold and Italics
"""

# ╔═╡ 8c3253f4-86c2-4b88-9850-f7daed87774d
md"""
This is **bold**

This is _italics_

This is **_bold and italics_**

"""

# ╔═╡ 40139e5d-3069-4a48-ab37-48849eac8ec6
md"""
#### Lists
"""

# ╔═╡ 939053b1-5c6b-4246-9102-d845e37c70a0
md"""
- This is my first item

- This is my second item

  - Two tabs give my first sub-item

  - My second

    - Four tabs give first sub-sub-item

    - Very easy

- My final item

1. My first numbered item

2. Whau, this is easy

"""

# ╔═╡ a1e2ea1a-8ffb-49e8-a22e-8be913a5074f
md"""
#### Writting with mathematical symbols
"""

# ╔═╡ cab5fa24-d103-4ce6-b979-80901b9a01a0
md"""

To write mathematical expressions in markdown, we only need to use the `$`. Start an expression with a `$` and end it with another `$`.

This is inline mathematics: $y=2x$.

This is displayed mathematics:

$$y=2x$$

This is mathematics using the power syntax: $y = 2x^3$

This is mathematics using the fraction syntax: $h = \dfrac{3}{4n}$

This is more elaborate mathematics: $z = \int_{a}^{b} x^2 dx$

This is ... lack of imagination, using greek symbols:

$$\lambda = \sum_{i=0}^{\infty} \alpha^i x_{i-1}$$
"""

# ╔═╡ f6162891-de97-4522-a919-4942158cb2a0
md"""
#### Tables
"""

# ╔═╡ b3c16511-06d5-476d-b1ce-b8b35d038819
md"""

| variable           | mean        | min       | median   | max      | 
|:--------------     |:-------:    |:--------: |:--------:|:--------:|           
| Students passed    | 14.9389     | 3.0       | 15.0     | 20       | 
| Midterm test       | 14.6172     | 8.85      | 14.8     | 19.7     | 
| Endterm test       | 15.8715     | 8.1       | 16.3     | 19.85    | 
"""

# ╔═╡ dca08890-c8af-4a31-b7cd-bf27e9bd91b7
md"""
!!! note "Exercise 1"
 
	- Open a new cell and turn it into markdown mode. Write in this cell the following text: `Today is a sunny day!`
	- Open another cell, turn it into markdown mode, and write the same text but in bold, and also in italics.
	- Open a new cell, turn it into markdown mode, and write the following: This is text and mathematics with a funny equation $y = xz/2$
	- Open a new cell, turn it into markdown mode, and write the following equation: $y = 3x^2$
	
"""

# ╔═╡ a0280f97-cf3d-4aaf-a013-45d957bd5700
md"""
!!! tip "Answer"

	Insert your answers below this box.	 

"""

# ╔═╡ 5b3d0a21-ab5c-4b41-8824-eae02427a14c
md"""
Today is a sunny day!

**Today is a sunny day!**

_Today is a sunny day!_

$y=xz/2$

$y=3x^2$
"""

# ╔═╡ dfc08cee-92a3-43dd-a4be-e66a7a3ae4d9
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 847ceec0-33cc-4cf4-a27f-ab7b85944529
md"""
## 3. Arithmetic operators
"""

# ╔═╡ 8f1bb867-1fa7-45e1-9806-70055ae22403
md"""

	+x	    unary plus	        the identity operation

	-x	    unary minus	      	maps values to their additive inverses

	x + y	binary plus	    	performs addition

	x - y	binary minus		performs subtraction

	x * y	times	        	performs multiplication

	x / y	divide	        	performs division

	x ÷ y	integer divide		x / y, truncated to an integer

	x \ y	inverse divide		equivalent to y / x

	x ^ y	power				raises x to the yth power

	x % y	remainder			equivalent to rem(x,y)'

"""

# ╔═╡ 4e908c09-f7eb-4698-9e9a-af1b7f7e5dc0
2+3

# ╔═╡ eea51a0b-cbe8-4fec-baec-ebc0601374d5
10^4

# ╔═╡ 731a0394-6dc6-44af-ada7-164fe8ed0c20
pepe = 10

# ╔═╡ 697cac78-ee49-48b7-9e7e-e6eace68cab9
rock = 20

# ╔═╡ abf827e7-0a9f-45e0-b89e-4b7ebaece273
mary = pepe * rock

# ╔═╡ b26522be-2c5e-4cd0-bfa2-eb20ff1f8070
md"""
`fiona = `$(@bind fiona Slider(-5.0:0.5:5.0, default=1.0, show_value=true))
"""

# ╔═╡ f01ee3cf-674b-41e2-8615-aebacb9ba384
paty = (fiona * mary , fiona^2 , fiona*mary , 10*fiona)

# ╔═╡ 26c4d01b-729e-4629-90a8-1e91247d0dff
md"""
!!! note "Exercise 2"
 
	What is the value of a variable called "zazu", given the following equation?

	$$zazu = 10 + rock^2 + \frac{pepe}{2}$$
"""

# ╔═╡ 44eadb1a-e4e2-4e62-a2b5-c29d11e9dc40
zazu = 10 + rock^2 + pepe/2

# ╔═╡ 9d6a33a0-72c5-4c36-b1be-fa17996ceda9
md"""
!!! tip "Answer"

	zazu = 415.	 	 

"""

# ╔═╡ b3742612-ebb6-495d-add9-f3287ce11b8d
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ d816ec4a-6478-4508-b7e3-22466a75c5cb
md"""
## 4. Ploting functions
"""

# ╔═╡ 4bd7ab99-9b66-4e54-97a1-9026554fb57f
begin
	x =  0.0 : 0.01 : 20.0
	#ana = 2 * x.^0.5
	ana = 2 * sin.(x) 	# the . is required here (says that sin is applied to element-by-element of x)
end;

# ╔═╡ 69e7ab97-6177-4bce-90b4-dea49addf5fd
begin
	fig1 = Plot(x, ana)
	#relayout!(fig1, Layout(hovermode="x", title_text="My fancy plot", title_x = 0.5, titlefont_size="17"))
	#fig1
end

# ╔═╡ 8fa56fa5-0e74-4340-98bf-de3870eeb4a1
noise = randn(500);

# ╔═╡ 2bfee414-f518-4b7f-902f-af86d7b53a4d
Plot(noise)

# ╔═╡ 12a07415-1077-4d6d-a414-897265ffe641
Plot(histogram(x=noise, opacity=0.75, nbinsx=50), Layout(bargap= 0.05))

# ╔═╡ 8d10cf9f-424d-4a6b-b039-c65ce4e9160f
md"""
!!! note "Exercise 3"
 
	In the following cell, produce a simple plot of the following function: 

	$$y=e^{ana}$$ 
	
	by using the following code:`Plot(x, exp.(ana))`
"""

# ╔═╡ 17c2ace2-75b6-4dc2-abdd-02a7d01dd608
Plot(x, exp.(ana))

# ╔═╡ b0ef4ed2-8236-4809-a508-74fee43f4bb2
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 9b622087-ad34-4f19-9929-cbdf0256eae0
md"""
## 5. Magic things we can do with Pluto
"""

# ╔═╡ b8b7733f-d8ea-48d9-a24a-a938dc83f641
md"""
### Fancy symbols for writing text and doing computation
"""

# ╔═╡ af94fe17-19e7-418d-90a8-6e76ecfbfb29
md"""
I have a 😺 that loves the 🌚, but not 🍍. And I can do write simple text or do computations with these adorable objects. 
"""

# ╔═╡ a1011eff-d753-437a-8139-925cef00f188
md"""
!!! tip
	To write special characters (like beta, for example) do the following: 
	- type: `\beta` and then click on `Tab` key to obtain β
	  - The firt time you do it, you have to click more than once on the `Tab` key. After that, clicking only once will be enough.
	- type: `\:smile` and then click on `Tab` to obtain 😄
"""

# ╔═╡ d4b64810-d65f-45ea-9037-bb2e57ff8811
β = 3

# ╔═╡ 70a5fdbe-17fa-4766-9c30-ab97113507b9
α = 10 * β

# ╔═╡ e1474fb7-6814-4294-b752-b0156db225f2
β^2

# ╔═╡ 8f9ad2ee-2876-4dcd-9892-7a0e8c962b27
begin # whenever a code cell contains more than 1 line, use the block begin ... end
	🍏 = 100
	😃 = 500
end

# ╔═╡ d79c25f1-153c-4ef3-824d-b688f666d2ac
🍏*🍏 / (2*😃)

# ╔═╡ 7eaf19e5-f205-485d-82a8-48fc1987df33
md"""
!!! note "Exercise 4"
 
	- Open a new cell and turn it into markdown mode. 
	- Write in this cell the following symbols (beta, psi, theta): β ψ θ
"""

# ╔═╡ edd2845f-2da7-49b8-a509-f2bfa391c7cb
md"""
β ψ θ
"""

# ╔═╡ c39ba17f-5894-4bc4-b6aa-f6f39947dc59
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 5b28585a-29ef-4de8-91a9-7dd9fd559515
md"""
### Play with sliders
"""

# ╔═╡ 1dfab317-6e41-4c11-a811-683f0fd897b4
md"""
!!! note "Exercise 5"
 
	Move the two sliders above (one slide at a time), and see what happens to the equilibrium in the plot above.
"""

# ╔═╡ d06b313f-d6c0-45bc-96f5-7514c9765f81
md"""
!!! tip "Answer"

	If we change the sliders, the equilibrium at point 1 will be changed.  	 	 

"""

# ╔═╡ 86249e98-7e54-4e42-a02f-ab5af6120194
md"""
!!! note "Exercise 6"
 
	Move the two sliders above (one slide at a time), and see what happens to the sinusoidal wave below.
"""

# ╔═╡ 6d6fd096-87de-46e2-92ce-9e6da7d1ebda
md"""
#### Calculate an integral
"""

# ╔═╡ 98fcb7b2-4d54-4119-a8ea-9545bea129e3
md"""
!!! note "Exercise 7"
 
	We can estimate the integral of a function $F:[a,b] \rightarrow \mathbb{R}$ using the midpoint rule approximation. Let us interactively estimate the $\int_0^1 F(x) d x$ to see the effect of changing the number of subintervals, in the case of:

	$$F(x)= - xsin(2\pi x)$$
"""

# ╔═╡ 4dc03487-e364-43e7-ba4a-e88ac9344089
begin
	Nbins_slider = @bind Nbins html"<input type=range min=1 max=400 step=1 value=20>"
		
	md"""Number of intervals: $Nbins_slider"""
end

# ╔═╡ fa2bb866-fac3-481c-837b-a8425fe84160
let
	F(x)=-x*sin(2π*x) 
	centres=1/(2*Nbins):1/Nbins:1
	midpointarea=round(sum(F.(centres))/Nbins, digits=6)
	
	trace01 = bar(x=centres, y=F.(centres), kind="bar", name = "Area")
	trace02 = scatter(x=centres, y= .- centres .* sin.(2 .* π .* centres), name = "F(x)")
		
	Plot([trace01, trace02], Layout(title_text="midpoint rule approximation with $Nbins intervals = $midpointarea"))
end

# ╔═╡ 72e56a5d-acf3-494d-b870-7806855eb74c
md"""
!!! tip "Answer"

	The solution is very, very close to 0.159157.  	 	 

"""

# ╔═╡ 09a44240-57a1-4064-9048-406cf4073cb9
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ cd0cd247-d257-49b1-a43b-f548a6062e36
md"""
### Playing with questions & answers 
"""

# ╔═╡ 987351ec-a397-4d67-a33f-967cd3464913
md"""
!!! note "Exercise 8"
 
	Table 1 shows some important macroeconomic aggregates, measured in **real terms**, for the US economy in 2013. The data in this table has been inserted into the notebook (see cell below). Answer the questions 👉 **a)**. 
"""

# ╔═╡ 5045c5ff-6ec3-4d4a-b076-5356c39c3355
md"""
$$\begin{aligned}
& \text {Table 1. Expenditure in the USA in 2013}\\
&\begin{array}{cllr}
\hline \hline \text {  } & \text{Code} & \text {Headings} & \text {Billions of dollars} \\
\hline  
1 & cons & \text{Private Consumption Expenditures} & 11400 &  \\
2 & inve & \text{Investment in Equipment and Structures} & 2600 &  \\
3 & stoc & \text{Inventories} & 80 &  \\
4 & gove & \text{Public Expenditures} & 3120 &  \\
5 & expo & \text{Exports} & 2250 &  \\
6 & impo & \text{Imports} & 3000 &  \\

\hline
\end{array}
\end{aligned}$$

"""

# ╔═╡ 4930d23c-d101-4c36-b2f6-d6b8ddca1dc0
# values passed into the notebook
cons = 11400 ; inve = 2600 ; stoc = 80 ; gove = 3120 ; expo = 2250 ; impo = 3000 ;

# ╔═╡ 30e9b082-c43d-4f5b-b5bc-c5038e121dc6
md"""
👉 **a)** Using Table 1, calculate GDP by the expenditure method. 
"""

# ╔═╡ a11ee5f4-6790-4a96-8ee0-3c6b7ef0198e
md"""
!!! hint

	According to the usual definition of GDP by the expenditure method, GDP is equal to the sum of  private consumption, plus public expenditure on goods and services, plus investment and inventories, plus exports minus imports.
"""

# ╔═╡ d2cd2672-83a0-4488-80e9-0c54c064b19d
gdp = cons + inve + stoc + gove + expo - impo

# ╔═╡ 1818b565-dbd0-438b-9a69-0815b8013675
md"""
!!! tip "Answer (a) "

	GDP was equal to 16450 billion dollars in 2013.

"""

# ╔═╡ 2b457972-3c5d-4421-90df-42c09d0d68c8
md"""
!!! note "Exercise 9"
 
	We can use Pluto to create multiple-choice questions with some pedagogic feedback. Try to answer the following question using the bullets below.

"""

# ╔═╡ afca4d98-7fe9-4123-b130-d3f5b3813db0
md"""
How much did the US GDP change in 2023, given the following facts:

- First Republic Bank shareholders lost 2 billion dollars on March 16 due to the bank's collapse.

- Microsoft decided to invest 10 billion dollars in the development of ChatGPT (OpenAI). 

- Amazon bought 100% of the capital of the company One Medical on February 22, 2023 for 4 billion dollars.


**a)** -- 2 billion + 10 billion + 4 billion.

**b)** + 4 billion.

**c)** + 10 billion + 4 billion.

**d)** + 10 billion.

**e)** None of the above.
"""

# ╔═╡ 570a64b5-f068-4d02-8196-97798a3f2d35
@bind q20 Radio(["a)","b)","c)","d)","e)"])

# ╔═╡ c2e708fd-57d9-4059-a126-d094a205bffd
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ f2956bed-9534-4c57-bfaa-fbef3c4af810
md"""
## 6. Downloading and plotting data
"""

# ╔═╡ 753355a9-0f46-41f0-b4c7-e6aaa328db07
md"""
!!! danger "Note"
	Students are not expected to know how to plot in this course, but they should be able to interact with the plots when required.
"""

# ╔═╡ 86c305dc-eed5-494f-ad3f-c222dbb4d48c
md"""
To download the data file attached to this notebook -- its name is _`ECB Data Inflation.csv`_ -- we should do as exemplified below. A CSV file is the most used format to deal with data and it means "comma separated values". This data file contains the inflation rate for the EuroZone as a whole and four individual countries. This data was obtained from [here](https://ec.europa.eu/eurostat/databrowser/view/prc_hicp_manr/default/table?lang=en).

```julia
name_my_data = CSV.read("ECB Data Inflation.csv", DataFrame) 	
```
"""

# ╔═╡ 14e220d5-8c83-4edf-97a6-03fd2d38391e
inflation = CSV.read("ECB Data Inflation.csv", DataFrame)

# ╔═╡ 973364f9-f009-47fd-b23c-63110d26b4d0
md"""
To summarize the data we downloaded, we should do:

```julia
describe(name_my_data)
```
"""

# ╔═╡ 7d101dd0-98e8-4907-b5a3-9cf8cbaf7529
describe(inflation)

# ╔═╡ a48efaec-6c05-420d-bfbe-7b0a70adf085
md"""
To plot a single column of `inflation` we can use, for example, the case of Portugal:
"""

# ╔═╡ e01ddc7c-6587-4847-a4ce-86d69978f7a6
begin
	fig5 = Plot(inflation.Months, inflation.Portugal)
end

# ╔═╡ 4e789f62-3e37-474b-a7c7-c5555be859f0
md"""
To plot more than one column, we can create a vector: `[entry1 entry2 entry3]`; in this example we have 3 columns, but we can use as many columns as we wish. In the case below we plot the columns associated with Germany and Portugal.
"""

# ╔═╡ 2e70e17c-123c-4602-a6db-c0b7110b99f7
begin
	fig6 = Plot(inflation.Months, [inflation.Germany  inflation.Portugal])
end

# ╔═╡ bf623a38-d3aa-453f-b1dc-1b89c40d2c52
md"""
We can add a _title_, the _names_ of the variables (columns), a _label_ to both axis, and a _hover_ functionality into our previous plot. It will look like this (the code is hidden: to show the code click on the little eye button on the left):
"""

# ╔═╡ 158b2712-96db-4652-9429-3e874d7fc3e7
begin
	fig7 = Plot(inflation.Months, [inflation.Germany  inflation.Portugal])
	
	restyle!(fig7, name = ["Germany", "Portugal"]) # names Germany and Portugal on variables 1 and 2
	
	relayout!(fig7, Layout(
		title_text = "Inflation in Germany and Portugal", title_x = 0.5, 
		hovermode = "x", yaxis_title = "Percentage points", 
		xaxis_title = "Monthly observations")) # introduces a title, y-axis label, and the hover-mode
	
	fig7 # Updates the plot
end

# ╔═╡ 5d9a6f46-1f6b-4802-bee9-1b7f236630a4
md"""

Furthermore, the plot, including all the variables in our data file and all the regular ingredients of a presentable figure (title, names, labels, and a hover), can be found below.

"""

# ╔═╡ fb315a6d-2322-4ca1-ae78-e0475d03f9b5
begin
	trace1_1 = scatter(;x = inflation.Months, y = inflation.EuroArea, 
				name = "EuroArea", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	marker_color = "Blue")
	
	trace1_2 = scatter(;x =inflation.Months, y = inflation.Portugal, 
				name = "Portugal", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Red")

	trace1_3 = scatter(;x = inflation.Months, y = inflation.Germany, 
				name = "Germany", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Gray")

	trace1_4 = scatter(;x = inflation.Months, y = inflation.Spain, 
				name = "Spain", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Purple")

	trace1_5 = scatter(;x = inflation.Months, y = inflation.France, 
				name = "France", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Orange")

	layout1_2 = Layout(;#font_size = 16,
					
			title_text = "Inflation in the EuroZone and Portugal", title_x=0.5,
		
			hovermode="x",		
		
            xaxis = attr(
               title = "Monthly obervations",
               #tickformat = "%Y",
               #hoverformat = "%Y-M%m",
               tick0 = "1997/01/01",
               dtick = "M60" ),
		
        	xaxis_range = [Date.(1997), Date.(2024)],
        	yaxis_title = "Percentage points",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	fig8 = Plot([trace1_1, trace1_2, trace1_3, trace1_4, trace1_5], layout1_2)
end

# ╔═╡ 2294234d-e562-473f-9317-32bb013b9cf7
md"""
!!! note "Exercise 10"
 
	We can also save one plot as a "png" file or an "svg" file (among many other formats). These files can be inserted later into another document (like a PowerPoint or a Word file). Their graphical quality will be much higher than by using the archaic "screen-capture" functionality. In the next cell, save "fig6" as a svg file by copying and running the code `savefig(fig6, "inflation.svg")`. It will be automatically saved in the same folder where this notebook is.

"""

# ╔═╡ d9c8e847-947f-447f-abb4-f8d88bf830a5
savefig(fig6, "inflation.svg")

# ╔═╡ 24236c6d-64cd-48be-bed8-8c149d1848a3
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 4109af60-6178-4148-8efe-890245077188
md"""
## 7. Using linear algebra 
"""

# ╔═╡ 87b575b9-89e4-4c8a-91b0-8e10651909cf
md"""
!!! warning

	In computation, there is a crucial difference between a `,` and a `.`. Therefore, do not forget that $1/2=0.5$, NOT $1/2=0,5$. If you mistakenly use a `,` to express decimal units, you will create a problem your computer does not know how to solve.

"""

# ╔═╡ 3a6c700f-148b-4fff-aa3e-7d20f3bc714a
A = [1 2 3 ; 4 1 6 ; 7 8 1]

# ╔═╡ c582f490-048b-4ecc-b3fe-918d623eaeae
inv(A) # the inverse of A

# ╔═╡ 09d8edd6-9d23-4af9-b8eb-3ded6c961621
transpose(A) # the transpose of A can also be written as A'

# ╔═╡ a23f4a73-bfd8-41ae-9b65-67c8e3e7bec0
B = [α ; β ; 10] # a vector 3x1

# ╔═╡ d93094ca-4912-406a-a573-83887ec40848
det(A) # the determinant of Bia

# ╔═╡ 38851f55-d1af-40f7-a660-62bceec4093a
A*B 

# ╔═╡ eaf75283-c222-46ef-a5ef-9bf9a6a94b75
A^3

# ╔═╡ a4f98b1b-cbb9-4dab-9e84-eba5fcd25c42
zak = randn(3, 3)

# ╔═╡ 3850481f-94b1-4e34-ac43-a346ec70b038
apple = A * zak


# ╔═╡ 291bfa24-a5f1-4505-83c1-b03f838b74de
md"""
!!! note "Exercise 11"
 
	Let us create a matrix of dimension (100x100), with all its elements as randomly generated numbers. Give this matrix a funny name Shrek: `shrek = randn(100,100)`

	Calculate the inverse of Shrek. How long does it take your computer to calculate such an inverse matrix?
"""

# ╔═╡ b95ebf4c-e27e-4768-9473-cb6346f7ef03
begin
	shrek = randn(100,100)
	inv(shrek)
end

# ╔═╡ 183d31a2-781a-4fb3-b6e8-c5f9f24ae8d8
md"""
!!! tip "Answer"

	It takes around 260μs to perform the inverse of shrek. 

"""

# ╔═╡ 839efccb-0a96-4cd1-b98f-981017eb2de2
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 6460431a-c536-4a0b-8da5-c9e855bddd5a
md"""
## 8. Solving a complicated model
"""

# ╔═╡ fbac12bb-3a2f-4767-ba4d-5786b2285a25
md"""
!!! note "Exercise 11"
 
	Consider the following prototype of a macroeconomic model:

	$\begin{aligned}
	& y_{1} = a + 0.5 y_{2} - 2.5 y_{3} \\
	& y_{2} = -2 y_{1} + 5 y_{3} + b \\
	& y_{3} = 10 y_{1} + 2 c \\
	& a = 10 \ , \ b=2 \ , \ c=5
	\end{aligned}$

	Let us solve this model using the package `NLsolve`. The first thing we should do is to pass the values of the variables with known values to the notebook: $a=10,b=2,c=5$: 

"""

# ╔═╡ ee99589f-9794-49e8-bccd-0a580f489860
a = 10 ; b = 2 ; c = 5 ;  		# the data is passed to the notebook

# ╔═╡ 3af336b8-decb-4608-bdfe-5ba43e0e3b43
md"""
The first thing we have to is to write our model as an homegeneous system (zeros on its left hand side)

$$\begin{aligned}
&0=a+0.5 y_2-2.5 y_3 -y_1 \\
&0=-2 y_1+5 y_3+b -y_2 \\
&0=10 y_1+2 c - y_3
\end{aligned}$$
"""

# ╔═╡ e99f49c6-8ecf-4519-868d-5b76ee13e658
md"""
Then we have to have to write down our problem according to the syntax of NLsolve: 

- give a name to our problem: let us name it as _king _ julien_

- define the variables in our problem: v[1], v[2], v[3] 

- define the equations in our problem: F[1], F[2], F[3] 
"""

# ╔═╡ 7b2a274a-ee89-4bd7-a61a-eb9c97c7b548
begin
	function king_julien!(F, v)
		
		y1 = v[1] 
        y2 = v[2] 
		y3 = v[3] 

        F[1] = a + 0.5y2 - 2.5y3 - y1
        F[2] = -2y1 + 5y3 + b - y2
		F[3] = 10y1 + 2c - y3 
			
	end
end

# ╔═╡ 59988d03-8580-4597-81da-225379f0298f
md"""
Finally, we should compute the solution to our problem according to the syntax of NLsolve: 

- give a name to the solution to our problem: let us name it as _my _ solution_

- call the NLsolve function that will solve the problem: _nlsolve_ 

- pass the problem to be solved (in this case is _king _ julien_), and give three initial guesses for the package to start the computation. For linear problems $[0.0 \ ; 0.0 \ ; 0.0]$ always works. The number of initial guesses must equal the number of unknowns in the model.
"""

# ╔═╡ 4852e7c0-6bdd-4f7e-8def-587db2fffd75
begin
	my_solution = nlsolve(king_julien!, [0.0 ; 0.0 ; 0.0])		# provides the solution
	my_solution.zero   		 									# simplifies (rounds up) the solution
end

# ╔═╡ a0468b27-1df3-4661-baec-e7adb42f281e
md"""
!!! tip "Answer"

	The solution is: y1 = 5.5, y2 = 316, y3 = 65.

"""

# ╔═╡ 02738dfd-8dc3-4396-be73-6242df0f5234
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ edbd2b1c-1452-41a6-962c-3434bd420020
md"""
## Auxiliary cells (please do not change them)
"""

# ╔═╡ c7190cc0-08d8-45c8-b5de-e5630e0ff00e
#html"""<style>
#main { max-width: 900px; align-self: flex-start;  margin-left: 100px;}
#"""

# ╔═╡ d9672a6f-97af-43d1-9a30-50495204fec8
 TableOfContents()

# ╔═╡ 37e34e48-c2a7-4ffd-ac97-1a6ecb44ea92
begin	
	# Demand	
	Ā3 = 8.0
	λ3 = 0.5
	m3 = 2.0
	ϕ3 = 0.2
	r̄3 = 2.0	
	# Supply	
	γ3  = 2.8
	Yᴾ3 = 14.8
	πᵉ3 = 2.0
	ρ3  = 0.0	
	# Intercepts (useful to simplify notation)	
	πd_max3 = (Ā3 / (λ3 * ϕ3)) - r̄3/λ3        # The AD curve y-axis' intercept
	πs_min3 = πᵉ3 - γ3 * Yᴾ3 + ρ3             # The AS curve y-axis' intercept
end;

# ╔═╡ 1bf47443-0ee4-482a-b637-8081620c144b
begin
	π_ZL3= -r̄3 / (1+λ3)
	πd_min3 = - Ā3 / ϕ3
	Y_ZL3 = Ā3 * m3 - (m3 *  ϕ3 * r̄3) / (1+λ3)
	ρ4 = -3.15
end;

# ╔═╡ 756b03e3-f745-48f6-bf5b-7c0f0d5d7c2f
begin
      struct TwoColumns{L, R}
    left::L
    right::R
      end
      
      function Base.show(io, mime::MIME"text/html", tc::TwoColumns)
          write(io, """<div style="display: flex;"><div style="flex: 48%;margin-right:2%;">""")
          show(io, mime, tc.left)
          write(io, """</div><div style="flex: 48%;">""")
          show(io, mime, tc.right)
          write(io, """</div></div>""")
      end
end

# ╔═╡ 9c051cae-e8c7-4dae-9878-67de9535e2bb
TwoColumns(
	
md"""

`ΔĀ = `$(@bind ΔĀ Slider(-1.0:0.05:1.0, default=0.0, show_value=true))

""",
	
md"""

`ΔYᴾ = `$(@bind ΔYᴾ Slider(-1.0:0.05:1.0, default=0.0, show_value=true))

"""
)

# ╔═╡ ebdcf47f-1328-4f35-b068-98f300708f06
begin
	Y3 = 13.8:0.01:15.6
	
	πd3 = πd_max3 .- ((1 ./(m3 .* ϕ3 .* λ3)) .* Y3) # AD function
	πs3 = πs_min3 .+ γ3 .*Y3             # AS function
	
	trace9_0 = scatter(; x = [14.8 .+ ΔYᴾ , 14.8 .+ ΔYᴾ] , y = [-5,10] , 
			mode = "line", line_width = "4", name = "Yp", line_color = "Orange")
	
	trace9_1 = scatter(; x = Y3, y = πd3, mode="lines" , line_color = "Blue", line_width = "3",
			name  = "AD")
	
	trace9_2 = scatter(; x = Y3, y = πs3, mode="lines" , line_color = "Red", line_width = "3",
					name  = "AS")
	
	trace9_3 = scatter(; x = [14.8], y = [2.0], text =["1"], 
					textposition = "top center", name ="Eq1", mode="markers+text", marker_size= "12",
					marker_color="Blue", textcolor = "Black",
	 				textfont = attr(family="sans serif", size=18, color="black"))


	πd_max3_ΔĀ = ((Ā3 .+ ΔĀ) ./ (λ3 .* ϕ3)) .- r̄3 ./ λ3 # New max value of the AD function
	
	πd3_ΔĀ = πd_max3_ΔĀ .- ((1 ./(m3 .* ϕ3 .* λ3)) .* Y3) # AD function
	
	
	trace9_4 = scatter(; x = Y3, y = πd3_ΔĀ, mode="lines" , line_color = "Magenta", 
					 line_width = "3", name  = "AD2")
		
	#trace9_5 = scatter(; x = [14.21], y = [2.947], text =["2"], 
	#				textposition = "top center", name ="Eq2", mode="markers+text", 
	#				marker_size= "12", marker_color="Magenta",
	#				textfont = attr(family="sans serif", size=16, color="black"))
	
	
	layout9_5 = Layout(;
					title_text ="Initial long-term equilibrium",
					title_x = 0.5,
					hovermode = "x",
                    xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
                    xaxis_range = [14.2, 15.4],	
                    yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					yaxis_range=[-0.8 , 5])

   fig0 = Plot([trace9_0, trace9_1, trace9_2, trace9_4, trace9_3], layout9_5)
	
	
end

# ╔═╡ 703ca971-86d4-40bf-9118-44468662953d
TwoColumns(
	
md"""

`frequency: `$(@bind frequency Slider(0.0:0.1:5.0, default=1.0, show_value=true))

""",
	
md"""

`damping:  `$(@bind damping Slider(-0.5:0.01:0.8, default=0.0, show_value=true))

"""
)

# ╔═╡ 593c6f61-9850-400e-917e-abeec3f06d97
begin
	τ=1:0.01:20
	Plot(τ, (exp.(.-damping * τ) .* sin.(frequency .* τ)))
end

# ╔═╡ c564fbd3-79a7-44eb-85fa-1e3ae565949d
function tchanã(ans::Union{Nothing,String},correct::String)	
	if typeof(ans) == Nothing
		md"""
		👆 *Please, select the correct answer.*
		
		---
		"""
	else
		if ans == correct
			md"""
			!!! tip "🎉 Corret!"

			---
			"""
		else	
			md"""
			!!! danger "👎 Wrong answer"
				
				Try again. **But it is better to study first!!!** 😡

			---
			"""
		end
	end
end;

# ╔═╡ 675932ae-9617-4ba4-be26-7a46745c5d44
tchanã(q20,"d)")

# ╔═╡ 60bfcb11-0250-4677-bea1-e47151000d41
md"### CSS code"

# ╔═╡ 98869c8a-3ed6-4042-ba8c-02b1d8225846
html""" <style>

	main {
		max-width: 1050px;
		padding: 5%;
        display: block !important;
        margin-left: 150px;
        margin-right: auto !important;
        user-select: auto !important;
         -webkit-user-select: auto !important;
         -moz-user-select: auto !important;
         -ms-user-select: auto !important;
	    /*counter-reset: h1;*/
    }
	
		form {
		text-align: center;
		width: fit-content;
		padding: 10px;
  		border: 5px solid #ebebeb;
  		margin: 1em 0 2.5em 0;
		border-radius: 25px;
  		background: #ebebeb;
		color:#1f1f1f;
	}
	form > div {
		display: inline-block;
		margin-left: 0.75em;
		margin-right: 0.75em;
		font-weight: bold;
	}
    /*pluto-cell>button,pluto-input>button,pluto-shoulder {
    display: none !important;
    }*/

</style>
"""

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
NLsolve = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
PeriodicalDates = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
PlotlyBase = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
PlotlyJS = "f0f68f2c-4968-5e81-91da-67840de0976a"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoTeachingTools = "661c6b06-c737-4d37-b85c-46df65de6f69"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
Statistics = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"

[compat]
CSV = "~0.10.4"
DataFrames = "~1.3.4"
HypertextLiteral = "~0.9.5"
NLsolve = "~4.5.1"
PeriodicalDates = "~2.0.0"
PlotlyBase = "~0.8.19"
PlotlyJS = "~0.18.12"
PlutoPlotly = "~0.4.4"
PlutoTeachingTools = "~0.2.14"
PlutoUI = "~0.7.40"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.10.0"
manifest_format = "2.0"
project_hash = "29680bad30b771f77144c728115ccc33c3353de5"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "c278dfab760520b8bb7e9511b968bf4ba38b7acc"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.2.3"

[[deps.Adapt]]
deps = ["LinearAlgebra", "Requires"]
git-tree-sha1 = "0fb305e0253fd4e833d486914367a2ee2c2e78d0"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "4.0.1"

    [deps.Adapt.extensions]
    AdaptStaticArraysExt = "StaticArrays"

    [deps.Adapt.weakdeps]
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[deps.ArrayInterface]]
deps = ["Adapt", "LinearAlgebra", "Requires", "SparseArrays", "SuiteSparse"]
git-tree-sha1 = "bbec08a37f8722786d87bedf84eae19c020c4efa"
uuid = "4fba245c-0d91-5ea0-9b3e-6abc04ee57a9"
version = "7.7.0"

    [deps.ArrayInterface.extensions]
    ArrayInterfaceBandedMatricesExt = "BandedMatrices"
    ArrayInterfaceBlockBandedMatricesExt = "BlockBandedMatrices"
    ArrayInterfaceCUDAExt = "CUDA"
    ArrayInterfaceGPUArraysCoreExt = "GPUArraysCore"
    ArrayInterfaceStaticArraysCoreExt = "StaticArraysCore"
    ArrayInterfaceTrackerExt = "Tracker"

    [deps.ArrayInterface.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    CUDA = "052768ef-5323-5732-b1bb-66c8b64840ba"
    GPUArraysCore = "46192b85-c4d5-4398-a991-12ede77f4527"
    StaticArraysCore = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
    Tracker = "9f7883ad-71c0-57eb-9f7f-b5c9e6d3789c"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[deps.AssetRegistry]]
deps = ["Distributed", "JSON", "Pidfile", "SHA", "Test"]
git-tree-sha1 = "b25e88db7944f98789130d7b503276bc34bc098e"
uuid = "bf4720bc-e11a-5d0c-854e-bdca1663c893"
version = "0.1.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[deps.BaseDirs]]
git-tree-sha1 = "4b41ad09c2307d5f24e36cd6f92eb41b218af22c"
uuid = "18cc8868-cbac-4acf-b575-c8ff214dc66f"
version = "1.2.1"

[[deps.BitFlags]]
git-tree-sha1 = "2dc09997850d68179b69dafb58ae806167a32b1b"
uuid = "d1d4a3ce-64b1-5f1a-9ba4-7e7e69966f35"
version = "0.1.8"

[[deps.Blink]]
deps = ["Base64", "Distributed", "HTTP", "JSExpr", "JSON", "Lazy", "Logging", "MacroTools", "Mustache", "Mux", "Pkg", "Reexport", "Sockets", "WebIO"]
git-tree-sha1 = "bc93511973d1f949d45b0ea17878e6cb0ad484a1"
uuid = "ad839575-38b3-5650-b840-f874b8c74a25"
version = "0.12.9"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "679e69c611fff422038e9e21e270c4197d49d918"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.12"

[[deps.CodeTracking]]
deps = ["InteractiveUtils", "UUIDs"]
git-tree-sha1 = "c0216e792f518b39b22212127d4a84dc31e4e386"
uuid = "da1fd8a2-8d9e-5ec2-8556-3022fb5608a2"
version = "1.3.5"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "59939d8a997469ee05c4b4944560a820f9ba0d73"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.4"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "67c1f244b991cad9b0aa4b7540fb758c2488b129"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.24.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "eb7f0f8307f71fac7c606984ea5fb2817275d6e4"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.4"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"
weakdeps = ["SpecialFunctions"]

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "fc08e5930ee9a4e03f84bfb5211cb54e7769758a"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.10"

[[deps.CommonSubexpressions]]
deps = ["MacroTools", "Test"]
git-tree-sha1 = "7b8a93dba8af7e3b42fecabf646260105ac373f7"
uuid = "bbf7d656-a473-5ed7-a52c-81e309532950"
version = "0.3.0"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "75bd5b6fc5089df449b5d35fa501c846c9b6549b"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.12.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.0.5+1"

[[deps.ConcurrentUtilities]]
deps = ["Serialization", "Sockets"]
git-tree-sha1 = "8cfa272e8bdedfa88b6aefbbca7c19f1befac519"
uuid = "f0e56b4a-5159-44fe-b623-3e5288b988bb"
version = "2.3.0"

[[deps.ConstructionBase]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "c53fc348ca4d40d7b371e71fd52251839080cbc9"
uuid = "187b0558-2788-49d3-abe0-74a17ed4e7c9"
version = "1.5.4"

    [deps.ConstructionBase.extensions]
    ConstructionBaseIntervalSetsExt = "IntervalSets"
    ConstructionBaseStaticArraysExt = "StaticArrays"

    [deps.ConstructionBase.weakdeps]
    IntervalSets = "8197267c-284f-5f27-9208-e0e47529a953"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "Future", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrettyTables", "Printf", "REPL", "Reexport", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "db2a9cb664fcea7836da4b414c3278d71dd602d2"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.3.6"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "ac67408d9ddf207de5cfa9a97e114352430f01ed"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.16"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DiffResults]]
deps = ["StaticArraysCore"]
git-tree-sha1 = "782dd5f4561f5d267313f23853baaaa4c52ea621"
uuid = "163ba53b-c6d8-5494-b064-1a9d43ac40c5"
version = "1.1.0"

[[deps.DiffRules]]
deps = ["IrrationalConstants", "LogExpFunctions", "NaNMath", "Random", "SpecialFunctions"]
git-tree-sha1 = "23163d55f885173722d1e4cf0f6110cdbaf7e272"
uuid = "b552c78f-8df3-52c6-915a-8e097449b14b"
version = "1.15.1"

[[deps.Distances]]
deps = ["LinearAlgebra", "Statistics", "StatsAPI"]
git-tree-sha1 = "66c4c81f259586e8f002eacebc177e1fb06363b0"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.11"

    [deps.Distances.extensions]
    DistancesChainRulesCoreExt = "ChainRulesCore"
    DistancesSparseArraysExt = "SparseArrays"

    [deps.Distances.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.ExceptionUnwrapping]]
deps = ["Test"]
git-tree-sha1 = "dcb08a0d93ec0b1cdc4af184b26b591e9695423a"
uuid = "460bff9d-24e4-43bc-9d9f-a8973cb893f4"
version = "0.1.10"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates", "Mmap", "Printf", "Test", "UUIDs"]
git-tree-sha1 = "9f00e42f8d99fdde64d40c8ea5d14269a2e2c1aa"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.21"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"

[[deps.FiniteDiff]]
deps = ["ArrayInterface", "LinearAlgebra", "Requires", "Setfield", "SparseArrays"]
git-tree-sha1 = "73d1214fec245096717847c62d389a5d2ac86504"
uuid = "6a86dc24-6348-571c-b903-95158fe2bd41"
version = "2.22.0"

    [deps.FiniteDiff.extensions]
    FiniteDiffBandedMatricesExt = "BandedMatrices"
    FiniteDiffBlockBandedMatricesExt = "BlockBandedMatrices"
    FiniteDiffStaticArraysExt = "StaticArrays"

    [deps.FiniteDiff.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[deps.Formatting]]
deps = ["Printf"]
git-tree-sha1 = "8339d61043228fdd3eb658d86c926cb282ae72a8"
uuid = "59287772-0a20-5a39-b81b-1366585eb4c0"
version = "0.4.2"

[[deps.ForwardDiff]]
deps = ["CommonSubexpressions", "DiffResults", "DiffRules", "LinearAlgebra", "LogExpFunctions", "NaNMath", "Preferences", "Printf", "Random", "SpecialFunctions"]
git-tree-sha1 = "cf0fe81336da9fb90944683b8c41984b08793dad"
uuid = "f6369f11-7733-5829-9624-2563aa707210"
version = "0.10.36"

    [deps.ForwardDiff.extensions]
    ForwardDiffStaticArraysExt = "StaticArrays"

    [deps.ForwardDiff.weakdeps]
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.FunctionalCollections]]
deps = ["Test"]
git-tree-sha1 = "04cb9cfaa6ba5311973994fe3496ddec19b6292a"
uuid = "de31a74c-ac4f-5751-b3fd-e18cd04993ca"
version = "0.5.0"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"

[[deps.HTTP]]
deps = ["Base64", "CodecZlib", "ConcurrentUtilities", "Dates", "ExceptionUnwrapping", "Logging", "LoggingExtras", "MbedTLS", "NetworkOptions", "OpenSSL", "Random", "SimpleBufferStream", "Sockets", "URIs", "UUIDs"]
git-tree-sha1 = "abbbb9ec3afd783a7cbd82ef01dcd088ea051398"
uuid = "cd3eb016-35fb-5094-929b-558a96fad6f3"
version = "1.10.1"

[[deps.Hiccup]]
deps = ["MacroTools", "Test"]
git-tree-sha1 = "6187bb2d5fcbb2007c39e7ac53308b0d371124bd"
uuid = "9fb69e20-1954-56bb-a84f-559cc56a8ff7"
version = "0.2.2"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "8b72179abc660bfab5e28472e019392b97d0985c"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.4"

[[deps.InlineStrings]]
deps = ["Parsers"]
git-tree-sha1 = "9cc2baf75c6d09f9da536ddf58eb2f29dedaf461"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[deps.InvertedIndices]]
git-tree-sha1 = "0dc7b50b8d436461be01300fd8cd45aa0274b038"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.0"

[[deps.IrrationalConstants]]
git-tree-sha1 = "630b497eafcc20001bba38a4651b327dcfc491d2"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.2"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JLLWrappers]]
deps = ["Artifacts", "Preferences"]
git-tree-sha1 = "7e5d6779a1e09a36db2a7b6cff50942a0a7d0fca"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.5.0"

[[deps.JSExpr]]
deps = ["JSON", "MacroTools", "Observables", "WebIO"]
git-tree-sha1 = "b413a73785b98474d8af24fd4c8a975e31df3658"
uuid = "97c1335a-c9c5-57fe-bc5d-ec35cebe8660"
version = "0.5.4"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.JuliaInterpreter]]
deps = ["CodeTracking", "InteractiveUtils", "Random", "UUIDs"]
git-tree-sha1 = "04663b9e1eb0d0eabf76a6d0752e0dac83d53b36"
uuid = "aa1ae85d-cabe-5617-a682-6adf51b2e16a"
version = "0.9.28"

[[deps.Kaleido_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "43032da5832754f58d14a91ffbe86d5f176acda9"
uuid = "f7e6163d-2fa5-5f23-b69c-1db539e41963"
version = "0.2.1+0"

[[deps.LaTeXStrings]]
git-tree-sha1 = "50901ebc375ed41dbf8058da26f9de442febbbec"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.3.1"

[[deps.Latexify]]
deps = ["Formatting", "InteractiveUtils", "LaTeXStrings", "MacroTools", "Markdown", "OrderedCollections", "Printf", "Requires"]
git-tree-sha1 = "f428ae552340899a935973270b8d98e5a31c49fe"
uuid = "23fbe1c1-3f47-55db-b15f-69d7ec21a316"
version = "0.16.1"

    [deps.Latexify.extensions]
    DataFramesExt = "DataFrames"
    SymEngineExt = "SymEngine"

    [deps.Latexify.weakdeps]
    DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
    SymEngine = "123dc426-2d89-5057-bbad-38513e3affd8"

[[deps.Lazy]]
deps = ["MacroTools"]
git-tree-sha1 = "1370f8202dac30758f3c345f9909b97f53d87d3f"
uuid = "50d2b5c4-7a5e-59d5-8109-a42b560f39c0"
version = "0.15.1"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.4.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.6.4+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[deps.LineSearches]]
deps = ["LinearAlgebra", "NLSolversBase", "NaNMath", "Parameters", "Printf"]
git-tree-sha1 = "7bbea35cec17305fc70a0e5b4641477dc0789d9d"
uuid = "d3d80556-e9d4-5f37-9878-2ab0fcc64255"
version = "7.2.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "7d6dd4e9212aebaeed356de34ccf262a3cd415aa"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.26"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[deps.LoggingExtras]]
deps = ["Dates", "Logging"]
git-tree-sha1 = "c1dd6d7978c12545b4179fb6153b9250c96b0075"
uuid = "e6f89c97-d47a-5376-807f-9c37f3926c36"
version = "1.0.3"

[[deps.LoweredCodeUtils]]
deps = ["JuliaInterpreter"]
git-tree-sha1 = "20ce1091ba18bcdae71ad9b71ee2367796ba6c48"
uuid = "6f1432cf-f94c-5a45-995e-cdbf5db27b0b"
version = "2.4.4"

[[deps.MIMEs]]
git-tree-sha1 = "65f28ad4b594aebe22157d6fac869786a255b7eb"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "0.1.4"

[[deps.MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "2fa9ee3e63fd3a4f7a9a4f4744a52f4856de82df"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.13"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[deps.MbedTLS]]
deps = ["Dates", "MbedTLS_jll", "MozillaCACerts_jll", "NetworkOptions", "Random", "Sockets"]
git-tree-sha1 = "c067a280ddc25f196b5e7df3877c6b226d390aaf"
uuid = "739be429-bea8-5141-9913-cc70e7f3736d"
version = "1.1.9"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.2+1"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "f66bdc5de519e8f8ae43bdc598782d35a25b1272"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.1.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.1.10"

[[deps.Mustache]]
deps = ["Printf", "Tables"]
git-tree-sha1 = "a7cefa21a2ff993bff0456bf7521f46fc077ddf1"
uuid = "ffc61752-8dc7-55ee-8c37-f3e9cdd09e70"
version = "1.0.19"

[[deps.Mux]]
deps = ["AssetRegistry", "Base64", "HTTP", "Hiccup", "MbedTLS", "Pkg", "Sockets"]
git-tree-sha1 = "0bdaa479939d2a1f85e2f93e38fbccfcb73175a5"
uuid = "a975b10e-0019-58db-a62f-e48ff68538c9"
version = "1.0.1"

[[deps.NLSolversBase]]
deps = ["DiffResults", "Distributed", "FiniteDiff", "ForwardDiff"]
git-tree-sha1 = "a0b464d183da839699f4c79e7606d9d186ec172c"
uuid = "d41bc354-129a-5804-8e4c-c37616107c6c"
version = "7.8.3"

[[deps.NLsolve]]
deps = ["Distances", "LineSearches", "LinearAlgebra", "NLSolversBase", "Printf", "Reexport"]
git-tree-sha1 = "019f12e9a1a7880459d0173c182e6a99365d7ac1"
uuid = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
version = "4.5.1"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "0877504529a3e5c3343c6f8b4c0381e57e4387e4"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.0.2"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.Observables]]
git-tree-sha1 = "7438a59546cf62428fc9d1bc94729146d37a7225"
uuid = "510215fc-4207-5dde-b226-833fc4488ee2"
version = "0.5.5"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.23+2"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+2"

[[deps.OpenSSL]]
deps = ["BitFlags", "Dates", "MozillaCACerts_jll", "OpenSSL_jll", "Sockets"]
git-tree-sha1 = "51901a49222b09e3743c65b8847687ae5fc78eb2"
uuid = "4d8831e6-92b7-49fb-bdf8-b643e874388c"
version = "1.4.1"

[[deps.OpenSSL_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl"]
git-tree-sha1 = "60e3045590bd104a16fefb12836c00c0ef8c7f8c"
uuid = "458c3c95-2e84-50aa-8efc-19380b2a3a95"
version = "3.0.13+0"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "13652491f6856acfd2db29360e1bbcd4565d04f1"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.5+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "dfdf5519f235516220579f949664f1bf44e741c5"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.6.3"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.PeriodicalDates]]
deps = ["Dates", "Printf", "RecipesBase"]
git-tree-sha1 = "e616941f8093e50a373e7d51875143213f82eca4"
uuid = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
version = "2.0.0"

[[deps.Pidfile]]
deps = ["FileWatching", "Test"]
git-tree-sha1 = "2d8aaf8ee10df53d0dfb9b8ee44ae7c04ced2b03"
uuid = "fa939f87-e72e-5be4-a000-7fc836dbe307"
version = "1.3.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.10.0"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "56baf69781fc5e61607c3e46227ab17f7040ffa2"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.19"

[[deps.PlotlyJS]]
deps = ["Base64", "Blink", "DelimitedFiles", "JSExpr", "JSON", "Kaleido_jll", "Markdown", "Pkg", "PlotlyBase", "REPL", "Reexport", "Requires", "WebIO"]
git-tree-sha1 = "bf9f8fbb4e76a621fe3a54a7281c5410b77ba015"
uuid = "f0f68f2c-4968-5e81-91da-67840de0976a"
version = "0.18.12"

[[deps.PlutoHooks]]
deps = ["InteractiveUtils", "Markdown", "UUIDs"]
git-tree-sha1 = "072cdf20c9b0507fdd977d7d246d90030609674b"
uuid = "0ff47ea0-7a50-410d-8455-4348d5de0774"
version = "0.0.5"

[[deps.PlutoLinks]]
deps = ["FileWatching", "InteractiveUtils", "Markdown", "PlutoHooks", "Revise", "UUIDs"]
git-tree-sha1 = "8f5fa7056e6dcfb23ac5211de38e6c03f6367794"
uuid = "0ff47ea0-7a50-410d-8455-4348d5de0420"
version = "0.1.6"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "BaseDirs", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "Reexport", "TOML"]
git-tree-sha1 = "58dcb661ba1e58a13c7adce77435c3c6ac530ef9"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.4.4"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoTeachingTools]]
deps = ["Downloads", "HypertextLiteral", "LaTeXStrings", "Latexify", "Markdown", "PlutoLinks", "PlutoUI", "Random"]
git-tree-sha1 = "89f57f710cc121a7f32473791af3d6beefc59051"
uuid = "661c6b06-c737-4d37-b85c-46df65de6f69"
version = "0.2.14"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "68723afdb616445c6caaef6255067a8339f91325"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.55"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "03b4c25b43cb84cee5c90aa9b5ea0a78fd848d2f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.0"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "00805cd429dcb4870060ff49ef443486c262e38e"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.1"

[[deps.PrettyTables]]
deps = ["Crayons", "Formatting", "Markdown", "Reexport", "Tables"]
git-tree-sha1 = "dfb54c4e414caa595a1f2ed759b160f5a3ddcba5"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "1.3.1"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[deps.RecipesBase]]
deps = ["PrecompileTools"]
git-tree-sha1 = "5c3d09cc4f31f5fc6af001c250bf1278733100ff"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.4"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.Revise]]
deps = ["CodeTracking", "Distributed", "FileWatching", "JuliaInterpreter", "LibGit2", "LoweredCodeUtils", "OrderedCollections", "Pkg", "REPL", "Requires", "UUIDs", "Unicode"]
git-tree-sha1 = "3fe4e5b9cdbb9bbc851c57b149e516acc07f8f72"
uuid = "295af30f-e4ad-537b-8983-00126c2a3abe"
version = "3.5.13"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "0e7508ff27ba32f26cd459474ca2ede1bc10991f"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.1"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[deps.Setfield]]
deps = ["ConstructionBase", "Future", "MacroTools", "StaticArraysCore"]
git-tree-sha1 = "e2cc6d8c88613c05e1defb55170bf5ff211fbeac"
uuid = "efcf1570-3423-57d1-acb7-fd33fddbac46"
version = "1.1.1"

[[deps.SimpleBufferStream]]
git-tree-sha1 = "874e8867b33a00e784c8a7e4b60afe9e037b74e1"
uuid = "777ac1f9-54b0-4bf8-805c-2214025038e7"
version = "1.1.0"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.10.0"

[[deps.SpecialFunctions]]
deps = ["IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "e2cfc4012a19088254b3950b85c3c1d8882d864d"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.3.1"

    [deps.SpecialFunctions.extensions]
    SpecialFunctionsChainRulesCoreExt = "ChainRulesCore"

    [deps.SpecialFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"

[[deps.StaticArraysCore]]
git-tree-sha1 = "36b3d696ce6366023a0ea192b4cd442268995a0d"
uuid = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
version = "1.4.2"

[[deps.Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.10.0"

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.SuiteSparse]]
deps = ["Libdl", "LinearAlgebra", "Serialization", "SparseArrays"]
uuid = "4607b0f0-06f3-5cda-b6b1-a6196a1729e9"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.2.1+1"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "LinearAlgebra", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "cb76cf677714c095e535e3501ac7954732aeea2d"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.11.1"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[deps.TranscodingStreams]]
git-tree-sha1 = "54194d92959d8ebaa8e26227dbe3cdefcdcd594f"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.10.3"
weakdeps = ["Random", "Test"]

    [deps.TranscodingStreams.extensions]
    TestExt = ["Test", "Random"]

[[deps.Tricks]]
git-tree-sha1 = "eae1bb484cd63b36999ee58be2de6c178105112f"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.8"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WebIO]]
deps = ["AssetRegistry", "Base64", "Distributed", "FunctionalCollections", "JSON", "Logging", "Observables", "Pkg", "Random", "Requires", "Sockets", "UUIDs", "WebSockets", "Widgets"]
git-tree-sha1 = "0eef0765186f7452e52236fa42ca8c9b3c11c6e3"
uuid = "0f1e0344-ec1d-5b48-a673-e5cf874b6c29"
version = "0.8.21"

[[deps.WebSockets]]
deps = ["Base64", "Dates", "HTTP", "Logging", "Sockets"]
git-tree-sha1 = "4162e95e05e79922e44b9952ccbc262832e4ad07"
uuid = "104b5d7c-a370-577a-8038-80a2059c5097"
version = "1.6.0"

[[deps.Widgets]]
deps = ["Colors", "Dates", "Observables", "OrderedCollections"]
git-tree-sha1 = "fcdae142c1cfc7d89de2d11e08721d0f2f86c98a"
uuid = "cc8bc4a8-27d6-5769-a93b-9d913e69aa62"
version = "0.6.6"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.8.0+1"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.52.0+1"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─3d4a86c6-9638-4e7c-86fb-f2764cd74c9f
# ╟─82090c38-1ecd-43eb-aeca-135d996b8e72
# ╟─76164b77-2e0b-4deb-8ef8-1b2263260e7a
# ╟─694d78c7-4762-4d40-8f9a-6c3d16916d4e
# ╟─554b2cc5-99e1-4eda-b080-67ab2eddac16
# ╠═b72215bb-8e3a-46e2-8817-271befe1178c
# ╟─31023dd0-f304-471e-a292-2a40915ebcbf
# ╟─7afb249a-ee89-4874-91fa-6da606948b2a
# ╟─072b3431-0b22-4801-a75e-25cd77a7d70d
# ╟─86f1359d-3637-4892-9720-e042fae6543a
# ╟─0fc80d2d-6519-4f17-83c4-25ab9b26d8ce
# ╟─16ecc97b-a401-4172-bcbb-b0baa156b53c
# ╟─564ff460-aa1f-4b9d-abf3-a024a13d96a3
# ╟─6307472b-3a3e-4d9a-82fd-25d295abaae1
# ╟─638ac59a-6fb2-447f-8856-0c6f74494e7b
# ╟─98f18fc3-0748-4ec7-a6a0-955e2ea22e19
# ╟─e81d6184-2b39-4205-83a8-9e2d7e5d41dd
# ╟─70af87bd-81b5-4aa0-b1f7-0639d1f09d0a
# ╟─2da4b605-2d9e-494a-810f-5d153dee7096
# ╟─8c3253f4-86c2-4b88-9850-f7daed87774d
# ╟─40139e5d-3069-4a48-ab37-48849eac8ec6
# ╟─939053b1-5c6b-4246-9102-d845e37c70a0
# ╟─a1e2ea1a-8ffb-49e8-a22e-8be913a5074f
# ╟─cab5fa24-d103-4ce6-b979-80901b9a01a0
# ╟─f6162891-de97-4522-a919-4942158cb2a0
# ╟─b3c16511-06d5-476d-b1ce-b8b35d038819
# ╟─dca08890-c8af-4a31-b7cd-bf27e9bd91b7
# ╟─a0280f97-cf3d-4aaf-a013-45d957bd5700
# ╟─5b3d0a21-ab5c-4b41-8824-eae02427a14c
# ╟─dfc08cee-92a3-43dd-a4be-e66a7a3ae4d9
# ╟─847ceec0-33cc-4cf4-a27f-ab7b85944529
# ╟─8f1bb867-1fa7-45e1-9806-70055ae22403
# ╠═4e908c09-f7eb-4698-9e9a-af1b7f7e5dc0
# ╠═eea51a0b-cbe8-4fec-baec-ebc0601374d5
# ╠═731a0394-6dc6-44af-ada7-164fe8ed0c20
# ╠═697cac78-ee49-48b7-9e7e-e6eace68cab9
# ╠═abf827e7-0a9f-45e0-b89e-4b7ebaece273
# ╟─b26522be-2c5e-4cd0-bfa2-eb20ff1f8070
# ╠═f01ee3cf-674b-41e2-8615-aebacb9ba384
# ╟─26c4d01b-729e-4629-90a8-1e91247d0dff
# ╠═44eadb1a-e4e2-4e62-a2b5-c29d11e9dc40
# ╟─9d6a33a0-72c5-4c36-b1be-fa17996ceda9
# ╟─b3742612-ebb6-495d-add9-f3287ce11b8d
# ╟─d816ec4a-6478-4508-b7e3-22466a75c5cb
# ╠═4bd7ab99-9b66-4e54-97a1-9026554fb57f
# ╠═69e7ab97-6177-4bce-90b4-dea49addf5fd
# ╠═8fa56fa5-0e74-4340-98bf-de3870eeb4a1
# ╠═2bfee414-f518-4b7f-902f-af86d7b53a4d
# ╠═12a07415-1077-4d6d-a414-897265ffe641
# ╟─8d10cf9f-424d-4a6b-b039-c65ce4e9160f
# ╠═17c2ace2-75b6-4dc2-abdd-02a7d01dd608
# ╟─b0ef4ed2-8236-4809-a508-74fee43f4bb2
# ╟─9b622087-ad34-4f19-9929-cbdf0256eae0
# ╟─b8b7733f-d8ea-48d9-a24a-a938dc83f641
# ╟─af94fe17-19e7-418d-90a8-6e76ecfbfb29
# ╟─a1011eff-d753-437a-8139-925cef00f188
# ╠═d4b64810-d65f-45ea-9037-bb2e57ff8811
# ╠═70a5fdbe-17fa-4766-9c30-ab97113507b9
# ╠═e1474fb7-6814-4294-b752-b0156db225f2
# ╠═8f9ad2ee-2876-4dcd-9892-7a0e8c962b27
# ╠═d79c25f1-153c-4ef3-824d-b688f666d2ac
# ╟─7eaf19e5-f205-485d-82a8-48fc1987df33
# ╟─edd2845f-2da7-49b8-a509-f2bfa391c7cb
# ╟─c39ba17f-5894-4bc4-b6aa-f6f39947dc59
# ╟─5b28585a-29ef-4de8-91a9-7dd9fd559515
# ╟─1dfab317-6e41-4c11-a811-683f0fd897b4
# ╟─9c051cae-e8c7-4dae-9878-67de9535e2bb
# ╟─ebdcf47f-1328-4f35-b068-98f300708f06
# ╟─d06b313f-d6c0-45bc-96f5-7514c9765f81
# ╟─86249e98-7e54-4e42-a02f-ab5af6120194
# ╟─703ca971-86d4-40bf-9118-44468662953d
# ╟─593c6f61-9850-400e-917e-abeec3f06d97
# ╟─6d6fd096-87de-46e2-92ce-9e6da7d1ebda
# ╟─98fcb7b2-4d54-4119-a8ea-9545bea129e3
# ╟─4dc03487-e364-43e7-ba4a-e88ac9344089
# ╟─fa2bb866-fac3-481c-837b-a8425fe84160
# ╟─72e56a5d-acf3-494d-b870-7806855eb74c
# ╟─09a44240-57a1-4064-9048-406cf4073cb9
# ╟─cd0cd247-d257-49b1-a43b-f548a6062e36
# ╟─987351ec-a397-4d67-a33f-967cd3464913
# ╟─5045c5ff-6ec3-4d4a-b076-5356c39c3355
# ╟─4930d23c-d101-4c36-b2f6-d6b8ddca1dc0
# ╟─30e9b082-c43d-4f5b-b5bc-c5038e121dc6
# ╟─a11ee5f4-6790-4a96-8ee0-3c6b7ef0198e
# ╠═d2cd2672-83a0-4488-80e9-0c54c064b19d
# ╟─1818b565-dbd0-438b-9a69-0815b8013675
# ╟─2b457972-3c5d-4421-90df-42c09d0d68c8
# ╟─afca4d98-7fe9-4123-b130-d3f5b3813db0
# ╟─570a64b5-f068-4d02-8196-97798a3f2d35
# ╟─675932ae-9617-4ba4-be26-7a46745c5d44
# ╟─c2e708fd-57d9-4059-a126-d094a205bffd
# ╟─f2956bed-9534-4c57-bfaa-fbef3c4af810
# ╟─753355a9-0f46-41f0-b4c7-e6aaa328db07
# ╟─86c305dc-eed5-494f-ad3f-c222dbb4d48c
# ╠═14e220d5-8c83-4edf-97a6-03fd2d38391e
# ╟─973364f9-f009-47fd-b23c-63110d26b4d0
# ╠═7d101dd0-98e8-4907-b5a3-9cf8cbaf7529
# ╟─a48efaec-6c05-420d-bfbe-7b0a70adf085
# ╠═e01ddc7c-6587-4847-a4ce-86d69978f7a6
# ╟─4e789f62-3e37-474b-a7c7-c5555be859f0
# ╠═2e70e17c-123c-4602-a6db-c0b7110b99f7
# ╟─bf623a38-d3aa-453f-b1dc-1b89c40d2c52
# ╟─158b2712-96db-4652-9429-3e874d7fc3e7
# ╟─5d9a6f46-1f6b-4802-bee9-1b7f236630a4
# ╟─fb315a6d-2322-4ca1-ae78-e0475d03f9b5
# ╟─2294234d-e562-473f-9317-32bb013b9cf7
# ╠═d9c8e847-947f-447f-abb4-f8d88bf830a5
# ╟─24236c6d-64cd-48be-bed8-8c149d1848a3
# ╟─4109af60-6178-4148-8efe-890245077188
# ╟─87b575b9-89e4-4c8a-91b0-8e10651909cf
# ╠═3a6c700f-148b-4fff-aa3e-7d20f3bc714a
# ╠═c582f490-048b-4ecc-b3fe-918d623eaeae
# ╠═09d8edd6-9d23-4af9-b8eb-3ded6c961621
# ╠═a23f4a73-bfd8-41ae-9b65-67c8e3e7bec0
# ╠═d93094ca-4912-406a-a573-83887ec40848
# ╟─38851f55-d1af-40f7-a660-62bceec4093a
# ╠═eaf75283-c222-46ef-a5ef-9bf9a6a94b75
# ╠═a4f98b1b-cbb9-4dab-9e84-eba5fcd25c42
# ╠═3850481f-94b1-4e34-ac43-a346ec70b038
# ╟─291bfa24-a5f1-4505-83c1-b03f838b74de
# ╠═b95ebf4c-e27e-4768-9473-cb6346f7ef03
# ╟─183d31a2-781a-4fb3-b6e8-c5f9f24ae8d8
# ╟─839efccb-0a96-4cd1-b98f-981017eb2de2
# ╟─6460431a-c536-4a0b-8da5-c9e855bddd5a
# ╟─fbac12bb-3a2f-4767-ba4d-5786b2285a25
# ╠═ee99589f-9794-49e8-bccd-0a580f489860
# ╟─3af336b8-decb-4608-bdfe-5ba43e0e3b43
# ╟─e99f49c6-8ecf-4519-868d-5b76ee13e658
# ╠═7b2a274a-ee89-4bd7-a61a-eb9c97c7b548
# ╟─59988d03-8580-4597-81da-225379f0298f
# ╠═4852e7c0-6bdd-4f7e-8def-587db2fffd75
# ╟─a0468b27-1df3-4661-baec-e7adb42f281e
# ╟─02738dfd-8dc3-4396-be73-6242df0f5234
# ╟─edbd2b1c-1452-41a6-962c-3434bd420020
# ╟─c7190cc0-08d8-45c8-b5de-e5630e0ff00e
# ╟─d9672a6f-97af-43d1-9a30-50495204fec8
# ╟─37e34e48-c2a7-4ffd-ac97-1a6ecb44ea92
# ╟─1bf47443-0ee4-482a-b637-8081620c144b
# ╟─756b03e3-f745-48f6-bf5b-7c0f0d5d7c2f
# ╟─c564fbd3-79a7-44eb-85fa-1e3ae565949d
# ╟─60bfcb11-0250-4677-bea1-e47151000d41
# ╟─98869c8a-3ed6-4042-ba8c-02b1d8225846
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
